package client;

import java.io.*;
import java.net.*;
import java.util.Date;

import common.Message;


public class SocketChatClient extends Thread {

    class ClientServiceThread extends Thread {
	private Socket s;

	public ClientServiceThread( Socket p ) {
	    s = p;
	}

	public void run( ) {
	    try {
		ObjectInputStream ois = new ObjectInputStream( s.getInputStream() );
		String cmd = (String) ois.readObject();
	    
		if ( cmd.equals("msg") ) {
		    Message msg = (Message) ois.readObject();
		    
		    SocketChatClient.this.receive( msg );
		} else {
		    System.err.println("Unknown command!");
		}
		ois.close();
		s.close();
	    } catch( Exception ioe ) {
		System.err.println("Error while decoding command!");
		ioe.printStackTrace();
	    }
	}
    }



    private static final String server_host = "gem.inf.ethz.ch";
    private static final int server_port = 6666;

    private InetAddress client_addr;
    private int client_port;

    public SocketChatClient( InetAddress caddr, int cport ) {
	try {
	    client_addr = caddr;
	    client_port = cport;

	    Socket s = new Socket(server_host, server_port);

	    ObjectOutputStream oos = new ObjectOutputStream( s.getOutputStream( ) );
	    oos.writeObject( "register" );
	    oos.writeObject( client_addr );
	    oos.writeInt( client_port );
	    oos.close();
	    s.close();
	} catch( IOException ioe ) {
	    System.err.println("Error while trying to send a message!");
	    ioe.printStackTrace();
	}
    }


    private synchronized void receive( Message m ) {
	System.out.println( m );
    }


    private synchronized void sendMessage( Message m ) {
	try {
	    Socket s = new Socket(server_host, server_port);
	    ObjectOutputStream oos = new ObjectOutputStream( s.getOutputStream( ) );
	    oos.writeObject( "bcast" );
	    oos.writeObject( m );
	    oos.close();
	    s.close();
	} catch( IOException ioe ) {
	    System.err.println("Error while trying to send a message!");
	    ioe.printStackTrace();
	}
    }

    private synchronized void deregister() {
	try {
	    Socket s = new Socket(server_host, server_port);
	    ObjectOutputStream oos = new ObjectOutputStream( s.getOutputStream( ) );
	    oos.writeObject( "deregister" );
	    oos.writeObject( client_addr );
	    oos.writeInt( client_port );
	    oos.close();
	    s.close();
	} catch( IOException ioe ) {
	    System.err.println("Error while trying to deregister!");
	    ioe.printStackTrace();
	}
    }

    public void run() {
	BufferedReader ir
	    = new BufferedReader(new InputStreamReader(System.in));
	String msgtext;
	Message msg;

	Runtime.getRuntime().addShutdownHook( new Thread() {
		public void run() {
		    deregister();
		}
	    } );

	while( true ) {
	    try {
		System.out.println("Enter a message sometime...");
		msgtext = ir.readLine();

		msg = new Message( client_addr + ":" + client_port,
				   new Date(), msgtext );
		System.out.println("Broadcasting message: " + msgtext );

		sendMessage( msg );
	    } catch( Exception e ) {
		System.err.println("Client had a problem...");
		e.printStackTrace();
	    }
	}
    }


    public static void main( String[] args ) {
	try {
	    // create a new server socket on any free port number
	    ServerSocket client_socket = new ServerSocket( 0 );
	    InetAddress localNetIP = InetAddress.getLocalHost();

	    SocketChatClient scc = new SocketChatClient( localNetIP, client_socket.getLocalPort() );
	    scc.start();

	    while ( true ) {
		Socket s = client_socket.accept( );
		scc.new ClientServiceThread( s ).start( );
	    }
	} catch( Exception e ) {
	    System.err.println("Client had a startup problem...");
	    e.printStackTrace();
	}

    }

}
