import java.util.*;
import java.lang.reflect.Method;

public class MethodObservable {

    // Mapping from target object to a set of methods to call
    private Map<Object, Set<Method>> calllist;

    public MethodObservable() {
	calllist = new HashMap<Object, Set<Method>>();
    }
 
    public void addObserver(Object t, Method m) {
	Set<Method> s;
	if( calllist.containsKey(t) ) {
	    s = calllist.get(t);
	    s.add( m );
	} else {
	    s = new HashSet<Method>();
	    s.add( m );
	}
	calllist.put( t, s );
    }

    public int countObservers() {
	return calllist.size();
    }

    public void deleteObserver(Object t, Method m) {
	if( calllist.containsKey(t) ) {
	    Set<Method> s = calllist.get(t);
	    s.remove( m );
	    if( s.isEmpty() ) {
		calllist.remove(t);
	    } else {
		calllist.put( t, s );
	    }
	}
    }

    public void deleteObservers() {
	calllist.clear();
    }

    public void notifyObservers() {
	notifyObservers( null );
    }

    public void notifyObservers(Object arg) {
	Object target = null;
	Method method = null;
	try {
	    for( Object t: calllist.keySet() ) {
		for( Method m: calllist.get(t) ) {
		    target = t; method = m;
		    method.invoke( target, this, arg );
		}
	    }
	} catch( Exception e ) {
	    handleException( target, method, e );
	}
    }

    protected void handleException( Object target, Method method,
				    Exception e ) {
	System.err.println("An exception occured!");
	e.printStackTrace();
    }
}
